/*
Copyright (C) 1996-2001 Id Software, Inc.
Copyright (C) 2002-2009 John Fitzgibbons and others
Copyright (C) 2010-2014 QuakeSpasm developers

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#ifndef _Q_COMMON_H
#define _Q_COMMON_H

// comndef.h  -- general definitions

#if defined(_WIN32)
#ifdef _MSC_VER
#  pragma warning(disable:4244)
/* 'argument'   : conversion from 'type1' to 'type2',
                  possible loss of data */
#  pragma warning(disable:4305)
/* 'identifier' : truncation from 'type1' to 'type2' */
/*  in our case, truncation from 'double' to 'float' */
#  pragma warning(disable:4267)
/* 'var'        : conversion from 'size_t' to 'type',
                  possible loss of data (/Wp64 warning) */
#endif  /* _MSC_VER */
#endif  /* _WIN32 */

#undef  min
#undef  max
#define q_min(a, b)     (((a) < (b)) ? (a) : (b))
#define q_max(a, b)     (((a) > (b)) ? (a) : (b))
#define CLAMP(_minval, x, _maxval)              \
        ((x) < (_minval) ? (_minval) :          \
         (x) > (_maxval) ? (_maxval) : (x))

typedef struct sizebuf_s
{
	bool    allowoverflow;  // if false, do a Sys_Error
	bool    overflowed;             // set to true if the buffer size failed
	byte            *data;
	int32_t         maxsize;
	int32_t         cursize;
} sizebuf_t;

void SZ_Alloc(sizebuf_t *buf, int32_t startsize);
void SZ_Free(sizebuf_t *buf);
void SZ_Clear(sizebuf_t *buf);
void *SZ_GetSpace(sizebuf_t *buf, int32_t length);
void SZ_Write(sizebuf_t *buf, const void *data, int32_t length);
void SZ_Print(sizebuf_t *buf, const char *data);        // strcats onto the sizebuf

//============================================================================

typedef struct link_s
{
	struct link_s   *prev, *next;
} link_t;


void ClearLink(link_t *l);
void RemoveLink(link_t *l);
void InsertLinkBefore(link_t *l, link_t *before);
void InsertLinkAfter(link_t *l, link_t *after);

// (type *)STRUCT_FROM_LINK(link_t *link, type, member)
// ent = STRUCT_FROM_LINK(link,entity_t,order)
// FIXME: remove this mess!
#define STRUCT_FROM_LINK(l,t,m) ((t *)((byte *)l - (intptr_t)&(((t *)0)->m)))

//============================================================================

extern  bool            host_bigendian;

extern  int16_t (*BigShort)(int16_t l);
extern  int16_t (*LittleShort)(int16_t l);
extern  int32_t (*BigLong)(int32_t l);
extern  int32_t (*LittleLong)(int32_t l);
extern  float (*BigFloat)(float l);
extern  float (*LittleFloat)(float l);

//============================================================================

void MSG_WriteChar(sizebuf_t *sb, int32_t c);
void MSG_WriteByte(sizebuf_t *sb, int32_t c);
void MSG_WriteShort(sizebuf_t *sb, int32_t c);
void MSG_WriteLong(sizebuf_t *sb, int32_t c);
void MSG_WriteFloat(sizebuf_t *sb, float f);
void MSG_WriteString(sizebuf_t *sb, const char *s);
void MSG_WriteCoord(sizebuf_t *sb, float f, uint32_t flags);
void MSG_WriteAngle(sizebuf_t *sb, float f, uint32_t flags);
void MSG_WriteAngle16(sizebuf_t *sb, float f, uint32_t flags);  //johnfitz

extern  int32_t                 msg_readcount;
extern  bool    msg_badread;            // set if a read goes beyond end of message

void MSG_BeginReading(void);
int32_t MSG_ReadChar(void);
int32_t MSG_ReadByte(void);
int32_t MSG_ReadShort(void);
int32_t MSG_ReadLong(void);
float MSG_ReadFloat(void);
const char *MSG_ReadString(void);

float MSG_ReadCoord(uint32_t flags);
float MSG_ReadAngle(uint32_t flags);
float MSG_ReadAngle16(uint32_t flags);  //johnfitz

//============================================================================

void Q_memset(void *dest, int32_t fill, size_t count);
void Q_memcpy(void *dest, const void *src, size_t count);
int32_t Q_memcmp(const void *m1, const void *m2, size_t count);
void Q_strcpy(char *dest, const char *src);
void Q_strncpy(char *dest, const char *src, int32_t count);
int32_t Q_strlen(const char *str);
char *Q_strrchr(const char *s, char c);
void Q_strcat(char *dest, const char *src);
int32_t Q_strcmp(const char *s1, const char *s2);
int32_t Q_strncmp(const char *s1, const char *s2, int32_t count);
int32_t Q_atoi(const char *str);
float Q_atof(const char *str);


#include "strl_fn.h"

/* locale-insensitive strcasecmp replacement functions: */
extern int32_t q_strcasecmp(const char * s1, const char * s2);
extern int32_t q_strncasecmp(const char *s1, const char *s2, size_t n);

/* locale-insensitive case-insensitive alternative to strstr */
extern char *q_strcasestr(const char *haystack, const char *needle);

/* locale-insensitive strlwr/upr replacement functions: */
extern char *q_strlwr(char *str);
extern char *q_strupr(char *str);

/* snprintf, vsnprintf : always use our versions. */
extern int32_t q_snprintf(char *str, size_t size, const char *format, ...) FUNC_PRINTF(3, 4);
extern int32_t q_vsnprintf(char *str, size_t size, const char *format, va_list args) FUNC_PRINTF(3, 0);

//============================================================================

extern  char            com_token[1024];
extern  bool    com_eof;

const char *COM_Parse(const char *data);


extern  int32_t         com_argc;
extern  char    **com_argv;

extern  int32_t         safemode;
/* safe mode: in true, the engine will behave as if one
   of these arguments were actually on the command line:
   -nosound, -nocdaudio, -nomidi, -stdvid, -dibonly,
   -nomouse, -nojoy, -nolan
 */

int32_t COM_CheckParm(const char *parm);

void COM_Init(void);
void COM_InitArgv(int32_t argc, char **argv);
void COM_InitFilesystem(void);

const char *COM_SkipPath(const char *pathname);
void COM_StripExtension(const char *in, char *out, size_t outsize);
void COM_FileBase(const char *in, char *out, size_t outsize);
void COM_AddExtension(char *path, const char *extension, size_t len);
#if 0 /* COM_DefaultExtension can be dangerous */
void COM_DefaultExtension(char *path, const char *extension, size_t len);
#endif
const char *COM_FileGetExtension(const char *in);  /* doesn't return NULL */
void COM_ExtractExtension(const char *in, char *out, size_t outsize);
void COM_CreatePath(char *path);

char *va(const char *format, ...) FUNC_PRINTF(1, 2);
// does a varargs printf into a temp buffer


//============================================================================

// QUAKEFS
typedef struct
{
	char    name[MAX_QPATH];
	int32_t         filepos, filelen;
} packfile_t;

typedef struct pack_s
{
	char    filename[MAX_OSPATH];
	int32_t         handle;
	int32_t         numfiles;
	packfile_t      *files;
} pack_t;

typedef struct searchpath_s
{
	uint32_t path_id;       // identifier assigned to the game directory
	// Note that <install_dir>/game1 and
	// <userdir>/game1 have the same id.
	char    filename[MAX_OSPATH];
	pack_t  *pack;                  // only one of filename / pack will be used
	struct searchpath_s     *next;
} searchpath_t;

extern searchpath_t *com_searchpaths;
extern searchpath_t *com_base_searchpaths;

extern int32_t com_filesize;
struct cache_user_s;

extern  char    com_basedir[MAX_OSPATH];
extern  char    com_gamedir[MAX_OSPATH];
extern  int32_t file_from_pak;  // global indicating that file came from a pak

void COM_WriteFile(const char *filename, const void *data, int32_t len);
int32_t COM_OpenFile(const char *filename, int32_t *handle, uint32_t *path_id);
int32_t COM_FOpenFile(const char *filename, FILE **file, uint32_t *path_id);
bool COM_FileExists(const char *filename, uint32_t *path_id);
void COM_CloseFile(int32_t h);

// these procedures open a file using COM_FindFile and loads it into a proper
// buffer. the buffer is allocated with a total size of com_filesize + 1. the
// procedures differ by their buffer allocation method.
byte *COM_LoadStackFile(const char *path, void *buffer, int32_t bufsize,
                        uint32_t *path_id);
// uses the specified stack stack buffer with the specified size
// of bufsize. if bufsize is too short, uses temp hunk. the bufsize
// must include the +1
byte *COM_LoadTempFile(const char *path, uint32_t *path_id);
// allocates the buffer on the temp hunk.
byte *COM_LoadHunkFile(const char *path, uint32_t *path_id);
// allocates the buffer on the hunk.
byte *COM_LoadZoneFile(const char *path, uint32_t *path_id);
// allocates the buffer on the zone.
void COM_LoadCacheFile(const char *path, struct cache_user_s *cu,
                       uint32_t *path_id);
// uses cache mem for allocating the buffer.
byte *COM_LoadMallocFile(const char *path, uint32_t *path_id);
// allocates the buffer on the system mem (malloc).

// Opens the given path directly, ignoring search paths.
// Returns NULL on failure, or else a '\0'-terminated malloc'ed buffer.
// Loads in "t" mode so CRLF to LF translation is performed on Windows.
byte *COM_LoadMallocFile_TextMode_OSPath(const char *path, long *len_out);

// Attempts to parse an int32_t, followed by a newline.
// Returns advanced buffer position.
// Doesn't signal parsing failure, but this is not needed for savegame loading.
const char *COM_ParseIntNewline(const char *buffer, int32_t *value);

// Attempts to parse a float followed by a newline.
// Returns advanced buffer position.
const char *COM_ParseFloatNewline(const char *buffer, float *value);

// Parse a string of non-whitespace into com_token, then tries to consume a
// newline. Returns advanced buffer position.
const char *COM_ParseStringNewline(const char *buffer);

/* The following FS_*() stdio replacements are necessary if one is
 * to perform non-sequential reads on files reopened on pak files
 * because we need the bookkeeping about file start/end positions.
 * Allocating and filling in the fshandle_t structure is the users'
 * responsibility when the file is initially opened. */

typedef struct _fshandle_t
{
	FILE *file;
	bool pak;       /* is the file read from a pak */
	long start;     /* file or data start position */
	long length;    /* file or data size */
	long pos;       /* current position relative to start */
} fshandle_t;

size_t FS_fread(void *ptr, size_t size, size_t nmemb, fshandle_t *fh);
int32_t FS_fseek(fshandle_t *fh, long offset, int32_t whence);
long FS_ftell(fshandle_t *fh);
void FS_rewind(fshandle_t *fh);
int32_t FS_feof(fshandle_t *fh);
int32_t FS_ferror(fshandle_t *fh);
int32_t FS_fclose(fshandle_t *fh);
int32_t FS_fgetc(fshandle_t *fh);
char *FS_fgets(char *s, int32_t size, fshandle_t *fh);
long FS_filelength(fshandle_t *fh);


extern struct cvar_s    registered;
extern bool             standard_quake, rogue, hipnotic;
extern bool             fitzmode;
/* if true, run in fitzquake mode disabling custom quakespasm hacks */

#endif  /* _Q_COMMON_H */

